@@ -1,4 +1,5 @@
-from typing import List, Type
+import asyncio
+from typing import List, Type, Coroutine, Callable, Any
 
 from sanic import json
 from sanic.exceptions import NotFound
@@ -8,11 +9,15 @@ from web_foundation.app.infrastructure.database.middleware import AccessDMW
 from web_foundation.app.infrastructure.database.models import AbstractDbModel
 from web_foundation.workers.io.http.chaining import InputContext
 
+ExternalCallback = Callable[
+    [AbstractDbModel | List[AbstractDbModel]], Coroutine[Any, Any, None]]
+
 
 async def exec_access(inc: InputContext,
                       model: Type[AbstractDbModel],
                       fetch_fields: List[str] = None,
-                      middleware: Type[AccessDMW] = AccessDMW
+                      middleware: Type[AccessDMW] = AccessDMW,
+                      external_callback: ExternalCallback = None
                       ):
     """
     Access to models
@@ -44,32 +49,28 @@ async def exec_access(inc: InputContext,
                     kwargs[ar] = val
 
                 retrieved_all, total = await middleware.read_all(model, inc.identity, limit, offset,
-                                                                fetch_fields=fetch_fields, **kwargs)
+                                                                 fetch_fields=fetch_fields, **kwargs)
+                if external_callback:
+                    asyncio.create_task(external_callback(retrieved_all))
                 res = [await model.values_dict() for model in retrieved_all]
-                return res
+                return res, total
             retrieved = await middleware.read(model, entity_id, inc.identity, fetch_fields=fetch_fields, **kwargs)
-            if retrieved:
-                result = await retrieved.values_dict()
-                return result
-            else:
+            if not retrieved:
                 raise NotFound()
 
         case "POST":
-            assert inc.dto
             retrieved = await middleware.create(model, inc.identity, inc.dto, **inc.r_kwargs)
-            if isinstance(retrieved, list):
-                return [await i.values_dict() for i in retrieved]
-            return await retrieved.values_dict()
 
         case "PATCH":
-            assert inc.dto
             entity_id = inc.r_kwargs.pop("entity_id")
-            assert entity_id
             retrieved = await middleware.update(model, entity_id, inc.identity, inc.dto, **inc.r_kwargs)
-            return await retrieved.values_dict()
 
         case "DELETE":
             entity_id = inc.r_kwargs.pop("entity_id")
-            assert entity_id
             retrieved = await middleware.delete(model, entity_id, inc.identity, **inc.r_kwargs)
-            return await retrieved.values_dict()
+
+    if external_callback:
+        asyncio.create_task(external_callback(retrieved))
+    if isinstance(retrieved, list):
+        return [await i.values_dict() for i in retrieved]
+    return await retrieved.values_dict()
